@@ -37,7 +37,7 @@
 #' @export
 summary.parameters <- function(object, ci.level = .9, ...) {
   if (length(ci.level) != 1 | !is.numeric(ci.level)) stop("ci must be a numeric vector of length 1. ")
-  if (!("parameters" %in% class(object))) stop("First argument has to be an object of class parameters. ")
+  if (!inherits(object, "parameters")) stop("First argument has to be an object of class parameters. ")
 
   UseMethod("summary.parameters")
 }
